#taken from https://docs.python.org/3/tutorial/classes.html
#
# CODE: Chiens2.py expands the example Chiens.py 
# GOAL: illustrate pitfalls associated with implicit shallow copy instead of explicity deep copy
#
# TO SEE THE EFFECT:
# launche ipython, import Chiens2 as ch, then
# a=ch.Dog('Wuff')
# a.measures
# a.add_measures(80., 15.)
# b=ch.Dog('Waff')
# b.measures
#
#     ---> what do you see?
#
# repeate (exit ipython and re-launch it), but instead of the shallow copy statement in __init__ (comment it out using an #)
#        self.measures = Dog.DogMeasures                  #implicit shallow copy, function add_measures will harm DogMeasures
# use an explicit deep copy statement (uncomment it by removing the #)
#        self.measures = copy.deepcopy(Dog.DogMeasures)   #explicit deep copy function add_measures will not harm DogMeasures
#
#     ---> what do you see now?
#
#------------------------------------------------------------------------------
import copy

class Dog:

    DogMeasures = [60., 5.] #a default dog-height of 60 cm and weight of 5 kg
    
    #initialization done upon creating an instance of class dog
    #instance: using the class Dog as blue print, create an object (instance) for one real dog
    def __init__(self, name):
        self.name = name    # name of dog
        self.tricks = []    # creates a new empty list for each dog
        self.measures = Dog.DogMeasures                  #implicit shallow copy, function add_measures will harm DogMeasures
        #self.measures = copy.deepcopy(Dog.DogMeasures)   #explicit deep copy function add_measures will not harm DogMeasures

    #once you have a concrete 'dog object' (instance), add a property of that concrete dog
    #'self' refers to the conrete dog / 'dog object' / instance of dog
    def add_trick(self, trick):
        self.tricks.append(trick)

    #or enable your dog to bark
    def bark(self, nbark):
        for ib in range(nbark):
            print("WUFF! ", end='')   #the , end='' prevents a newline character being printed after each WUFF
        print("\n")                   #end the barking with an extra new line characther
        
    #change the dog measures
    def add_measures(self, height, weight):
        self.measures[0] = height
        self.measures[1] = weight

